//spezifisch fr Arduino Giga R1 geschrieben...
#include <Arduino_USBHostMbed5.h>
#include <DigitalOut.h>
#include <FATFileSystem.h>
#include <SPI.h>
#include <WiFi.h>
#include <Keypad.h>
#include <LiquidCrystal.h>


USBHostMSD msd;
mbed::FATFileSystem usb("usb");
mbed::DigitalOut otg(PB_8, 1);

const int pinRedLED = 86;   // Rot
const int pinGreenLED = 87; // Grn
const int pinBlueLED = 88; // Blau

WiFiServer server(80);

// Keypad und LCD Initialisierung
const byte ROWS = 4; //vier Reihen
const byte COLS = 4; //vier Spalten
char hexaKeys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};
byte rowPins[ROWS] = {32, 34, 36, 38}; //verbinde mit den Reihen-Pinouts des Keypads
byte colPins[COLS] = {40, 42, 44, 46}; //verbinde mit den Spalten-Pinouts des Keypads
const int rs = 49, en = 51, d4 = 33, d5 = 35, d6 = 37, d7 = 39;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);
Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);


//berprfung
String inputString = ""; // Zum Speichern der Tasteneingaben
String correctString = ""; // Der zu berprfende String

void setup() {
  Serial.begin(9600);
  pinMode(PA_15, OUTPUT);
  digitalWrite(PA_15, HIGH);

  while (!msd.connect()) {
    delay(1000);
  }

  int err = usb.mount(&msd);
  if (err) {
    Serial.println("Error mounting USB device");
    while (1);
  }
  Serial.println("Mounting USB device... done");

  pinMode(pinRedLED, OUTPUT);
  pinMode(pinGreenLED, OUTPUT);
  pinMode(pinBlueLED, OUTPUT);
  digitalWrite(pinRedLED, LOW);

  char ssid[32] = "";
  char pass[32] = "";
  FILE *wifiFile = fopen("/usb/WiFi.txt", "r");
  if (wifiFile) {
    fscanf(wifiFile, "%31s\n%31s", ssid, pass);
    fclose(wifiFile);
  } else {
    strcpy(ssid, "SSID");
    strcpy(pass, "PASSWORD");
    FILE *newWifiFile = fopen("/usb/WiFi.txt", "w");
    if (newWifiFile) {
      fprintf(newWifiFile, "%s\n%s", ssid, pass);
      fclose(newWifiFile);
    }
  }

  int status = WL_IDLE_STATUS;
  while (status != WL_CONNECTED) {
    status = WiFi.begin(ssid, pass);
    delay(10000);
  }
  server.begin();
  digitalWrite(pinGreenLED, LOW);
  digitalWrite(pinRedLED, HIGH);
  digitalWrite(pinBlueLED, HIGH);
  Serial.println("Connected Succesfully!");
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // LCD Initialisierung
  analogWrite(2, 50); // Kontrast LCD einstellen
  lcd.begin(16, 2);
  lcd.print("IP: ");
  lcd.setCursor(0, 1);
  lcd.print(ip);
  delay(10000); // Warte 10 Sekunden
  lcd.clear();
  lcd.print("PIN eingeben:");
  lcd.setCursor(0, 1);

  // Lese die korrekten PINs aus der Datei PINs.txt
  FILE *pinFile = fopen("/usb/PINs.txt", "r");
  if (pinFile) {
    char line[32];
    while (fgets(line, sizeof(line), pinFile)) {
      correctString = String(line);
      correctString.trim(); // Entferne fhrende und abschlieende Leerzeichen
      // berprfe die PIN hier
    }
    fclose(pinFile);
  }
}


void loop() {
  WiFiClient client = server.available();
  if (client) {
    boolean currentLineIsBlank = true;
    String request = "";
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        if (c == '\n' && currentLineIsBlank) {
          if (request.indexOf("/generierePIN ") != -1) {
            String pin = generatePIN();
            // Schreibe die generierte PIN direkt in die Datei PINs.txt auf dem USB-Stick
            FILE *pinFile = fopen("/usb/PINs.txt", "a+");
            if (pinFile) {
              fprintf(pinFile, "%s\n", pin.c_str());
              fclose(pinFile);
            }
            client.println("HTTP/1.1 200 OK");
            client.println("Content-Type: text/html");
            client.println("Connection: close");
            client.println();
            client.println("<!DOCTYPE HTML>");
            client.println("<html>");
            client.println("Deine generierte PIN: " + pin);
            client.println("</html>");
          } else {
            client.println("HTTP/1.1 404 Not found");
            client.println("Content-Type: text/html");
            client.println("Connection: close");
            client.println();
            client.println("<!DOCTYPE HTML>");
            client.println("<html>");
            client.println("404 Not Found");
            client.println("</html>");
          }
          break;
        }

        if (c == '\n') {
          currentLineIsBlank = true;
        } else if (c != '\r') {
          currentLineIsBlank = false;
        }
        request += c;
      }
    }
    delay(1);
    client.stop();
  }

  char customKey = customKeypad.getKey();
  if (customKey) {
    if (customKey == '#') {
      // berprfe, ob die eingegebene PIN in den korrekten PINs enthalten ist
      bool pinAccepted = false;
      FILE *pinFile = fopen("/usb/PINs.txt", "r");
      if (pinFile) {
        char line[32];
        while (fgets(line, sizeof(line), pinFile)) {
          String correctPin = String(line);
          correctPin.trim();
          if (inputString == correctPin) {
            pinAccepted = true;
            break; // Beende die Schleife, wenn eine korrekte PIN gefunden wurde
          }
        }
        fclose(pinFile);
      }

      if (pinAccepted) {
        Serial.println("akzeptiert");
        lcd.clear();
        lcd.print("akzeptiert!");
        delay(1000);
        lcd.clear();
        lcd.print("PIN eingeben:");
        lcd.setCursor(0, 1);
      } else {
        Serial.println("nicht akzeptiert");
        lcd.clear();
        lcd.print("nicht akzeptiert!");
        delay(1000);
        lcd.clear();
        lcd.print("PIN eingeben:");
        lcd.setCursor(0, 1);
      }
      inputString = ""; // Setze den Eingabestring zurck
    } else if (customKey == '*') {
      // Setze den Eingabestring zurck
      inputString = "";
      Serial.println("Eingabe zurckgesetzt");
      lcd.clear();
      lcd.print("Eingabe zurueckgesetzt ");
      delay(1000);
      lcd.clear();
      lcd.print("PIN eingeben:");
      lcd.setCursor(0, 1);
    } else {
      // Fge die gedrckte Taste zum Eingabestring hinzu
      inputString += customKey;
      Serial.print(customKey); // Zeige die gedrckte Taste an
      lcd.print(customKey);
    }
  }
  delay(1);
}
String generatePIN() {
  const char validCharacters[] = "ABCD0123456789";
  String pin = "";
  for (int i = 0; i < 6; i++) {
    int index = random(0, sizeof(validCharacters) - 1);
    pin += validCharacters[index];
  }
  return pin;
}

void ledblink(int pinLED) {
  digitalWrite(pinLED, HIGH);
  delay(500);
  digitalWrite(pinLED, LOW);
}
